import { AllowedComponentProps } from 'vue';
import { Component } from 'vue';
import { DefineSetupFnComponent } from 'vue';
import { IfAny } from '@vue/shared';
import { InputHTMLAttributes } from 'vue';
import { PublicProps } from 'vue';
import { Ref } from 'vue';
import { SlotsType } from 'vue';
import { TypedSchema } from '../../packages/utils/types';
import { UnwrapRef } from 'vue';
import { VNodeProps } from 'vue';

export declare type ComponentProps<C extends Component> = C extends new (...args: any) => any ? Omit<InstanceType<C>['$props'], keyof VNodeProps | keyof AllowedComponentProps> : never;

export declare const createInput: <T, K extends Record<string, any> = Record<string, any>>(component: Component, options?: CreateInputOptions) => DefineSetupFnComponent<T & FieldType<K>, `update:${string}`[], {}, T & FieldType<K> & {
[x: `onUpdate:${string}`]: ((...args: any[]) => any) | undefined;
}, PublicProps>;

declare type CreateInputOptions = {
    default?: any;
    modelKey?: string;
    useModelKeyAsState?: boolean;
    defaultValueKey?: any;
    componentName?: string;
};

declare const Error_2: DefineSetupFnComponent<    {
errorFor: string | number;
}, {}, SlotsType<    {
default: {
error: any;
};
}>, {
errorFor: string | number;
} & {}, PublicProps>;
export { Error_2 as Error }

declare type ExtractValue<T, K extends string> = K extends `${infer Root}.${infer Rest}` ? Root extends keyof T ? ExtractValue<T[Root], Rest> : Root extends `${infer ArrayRoot}[${number}]` ? ArrayRoot extends keyof T ? T[ArrayRoot] extends (infer U)[] ? ExtractValue<U, Rest> : never : never : never : K extends `${infer Root}[${number}]` ? Root extends keyof T ? T[Root] extends (infer U) ? UnwrapArray<U> : never : never : K extends keyof T ? UnwrapArray<T[K]> : never;

export declare const Field: DefineSetupFnComponent<FieldType<Record<string, any>>, any, SlotsType<    {
default: {
field: {
value: any;
};
error: any;
};
}>, FieldType<Record<string, any>> & ({} | {
[x: `on${Capitalize<any>}`]: ((...args: any[]) => any) | undefined;
} | {
[x: `on${Capitalize<string>}`]: ((...args: any[] | unknown[]) => any) | undefined;
}), PublicProps>;

export declare const FieldArray: DefineSetupFnComponent<    {
name: string | number;
error?: any;
ignore?: boolean;
initialValues?: any[];
preserve?: boolean;
default?: any[];
}, "update:modelValue"[], SlotsType<    {
default: {
fields: {
id: number;
}[];
add: () => void;
remove: (idx: number) => void;
error: any;
};
}>, {
name: string | number;
error?: any;
ignore?: boolean;
initialValues?: any[];
preserve?: boolean;
default?: any[];
} & {
"onUpdate:modelValue"?: ((...args: any[]) => any) | undefined;
}, PublicProps>;

declare type FieldType<T extends Record<string, any>> = {
    [K in GetKeys<T>]: {
        name: K;
        default?: ExtractValue<T, K>;
    } & {
        error?: any;
        ignore?: boolean;
        trueValue?: any;
        modelValue?: any;
        falseValue?: any;
        preserve?: boolean;
        as?: 'input' | 'select';
    } & InputHTMLAttributes;
}[GetKeys<T>];

declare const Form: DefineSetupFnComponent<    {
enctype?: "application/x-www-form-urlencoded" | "multipart/form-data";
validationSchema?: any;
action?: string;
initialValues?: {
[x: string]: any;
} | undefined;
name?: string;
preserve?: boolean;
onValueChange?: (value?: any) => void;
onSubmit?: (value?: any, $event?: SubmitEvent) => void | Promise<any>;
}, ("submit" | "value-change")[], SlotsType<    {
default: {
values: Record<string, any>;
errors: any;
};
}>, {
enctype?: "application/x-www-form-urlencoded" | "multipart/form-data";
validationSchema?: any;
action?: string;
initialValues?: {
[x: string]: any;
} | undefined;
name?: string;
preserve?: boolean;
onValueChange?: (value?: any) => void;
onSubmit?: (value?: any, $event?: SubmitEvent) => void | Promise<any>;
} & {
onSubmit?: ((...args: any[]) => any) | undefined;
"onValue-change"?: ((...args: any[]) => any) | undefined;
}, PublicProps>;
export { Form }
export { Form as FormifyForm }

declare type FormOptions<T extends Record<string, any>> = {
    initialValues?: T extends Record<string, any> ? RecursivePartial<T> : Record<string, any>;
    schema?: TypedSchema<any, T>;
};

export declare type FormType<T extends Record<string, any> = Record<string, any>> = {
    values: T;
    errors: T;
    isSubmitting: boolean;
    setError: (name: GetKeys<T>, error: any) => void;
    updateField: (name: GetKeys<T>, value: any) => void;
    reset: () => void;
    flush: () => void;
};

declare type GetKeys<T extends Record<string, any>> = keyof {
    [K in keyof T as (T[K] extends (any | undefined)[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}[]` | `${K & string}[${number}]` | `${K & string}` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`)]: any;
};

declare type GetNestedArray<T> = T extends object ? {
    [K in keyof T]: T[K] extends (infer U)[] ? `${K & string}[]${U extends object ? `.${GetNestedArray<U>}` : ''}` | `${K & string}[${number}]${U extends object ? `.${GetNestedArray<U>}` : ''}` | `${K & string}` | `${K & string}[${number}]` : `${K & string}${T[K] extends object ? `.${GetNestedArray<T[K]>}` : `${GetNestedArray<T[K]>}`}`;
}[keyof T] : '';

declare type RecursivePartial<T> = {
    [P in keyof T]?: T[P] extends Record<string, any> ? RecursivePartial<T[P]> : T[P];
};

declare type UnwrapArray<T> = T extends (infer U)[] ? U : T;

export declare const useForm: <T extends Record<string, any>>(opt?: FormOptions<T>) => {
    Form: DefineSetupFnComponent<    {
    enctype?: "application/x-www-form-urlencoded" | "multipart/form-data";
    validationSchema?: any;
    action?: string;
    initialValues?: { [P in keyof T]?: (T[P] extends Record<string, any> ? T[P] extends infer T_1 ? { [P_1 in keyof T_1]?: (T[P][P_1] extends Record<string, any> ? T[P][P_1] extends infer T_2 ? { [P_2 in keyof T_2]?: (T[P][P_1][P_2] extends Record<string, any> ? T[P][P_1][P_2] extends infer T_3 ? { [P_3 in keyof T_3]?: (T[P][P_1][P_2][P_3] extends Record<string, any> ? T[P][P_1][P_2][P_3] extends infer T_4 ? { [P_4 in keyof T_4]?: (T[P][P_1][P_2][P_3][P_4] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4] extends infer T_5 ? { [P_5 in keyof T_5]?: (T[P][P_1][P_2][P_3][P_4][P_5] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5] extends infer T_6 ? { [P_6 in keyof T_6]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6] extends infer T_7 ? { [P_7 in keyof T_7]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7] extends infer T_8 ? { [P_8 in keyof T_8]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8] extends infer T_9 ? { [P_9 in keyof T_9]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9] extends infer T_10 ? { [P_10 in keyof T_10]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9][P_10] extends Record<string, any> ? any : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9][P_10]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4]) | undefined; } : never : T[P][P_1][P_2][P_3]) | undefined; } : never : T[P][P_1][P_2]) | undefined; } : never : T[P][P_1]) | undefined; } : never : T[P]) | undefined; } | undefined;
    name?: string;
    preserve?: boolean;
    onValueChange?: (value?: any) => void;
    onSubmit?: (value?: any, $event?: SubmitEvent) => void | Promise<any>;
    }, ("submit" | "value-change")[], SlotsType<    {
    default: {
    values: T;
    errors: any;
    };
    }>, {
    enctype?: "application/x-www-form-urlencoded" | "multipart/form-data";
    validationSchema?: any;
    action?: string;
    initialValues?: { [P in keyof T]?: (T[P] extends Record<string, any> ? T[P] extends infer T_1 ? { [P_1 in keyof T_1]?: (T[P][P_1] extends Record<string, any> ? T[P][P_1] extends infer T_2 ? { [P_2 in keyof T_2]?: (T[P][P_1][P_2] extends Record<string, any> ? T[P][P_1][P_2] extends infer T_3 ? { [P_3 in keyof T_3]?: (T[P][P_1][P_2][P_3] extends Record<string, any> ? T[P][P_1][P_2][P_3] extends infer T_4 ? { [P_4 in keyof T_4]?: (T[P][P_1][P_2][P_3][P_4] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4] extends infer T_5 ? { [P_5 in keyof T_5]?: (T[P][P_1][P_2][P_3][P_4][P_5] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5] extends infer T_6 ? { [P_6 in keyof T_6]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6] extends infer T_7 ? { [P_7 in keyof T_7]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7] extends infer T_8 ? { [P_8 in keyof T_8]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8] extends infer T_9 ? { [P_9 in keyof T_9]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9] extends Record<string, any> ? T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9] extends infer T_10 ? { [P_10 in keyof T_10]?: (T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9][P_10] extends Record<string, any> ? any : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9][P_10]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8][P_9]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7][P_8]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6][P_7]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5][P_6]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4][P_5]) | undefined; } : never : T[P][P_1][P_2][P_3][P_4]) | undefined; } : never : T[P][P_1][P_2][P_3]) | undefined; } : never : T[P][P_1][P_2]) | undefined; } : never : T[P][P_1]) | undefined; } : never : T[P]) | undefined; } | undefined;
    name?: string;
    preserve?: boolean;
    onValueChange?: (value?: any) => void;
    onSubmit?: (value?: any, $event?: SubmitEvent) => void | Promise<any>;
    } & {
    onSubmit?: ((...args: any[]) => any) | undefined;
    "onValue-change"?: ((...args: any[]) => any) | undefined;
    }, PublicProps>;
    Field: DefineSetupFnComponent<FieldType<T>, any, SlotsType<    {
    default: {
    field: {
    value: any;
    };
    error: any;
    };
    }>, FieldType<T> & ({} | {
    [x: `on${Capitalize<any>}`]: ((...args: any[]) => any) | undefined;
    } | {
    [x: `on${Capitalize<string>}`]: ((...args: any[] | unknown[]) => any) | undefined;
    }), PublicProps>;
    FieldArray: DefineSetupFnComponent<    {
    name: keyof { [K in keyof T as T[K] extends any[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}` | `${K & string}[]` | `${K & string}[${number}]` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`]: any; };
    error?: any;
    ignore?: boolean;
    initialValues?: any[];
    preserve?: boolean;
    default?: any[];
    }, "update:modelValue"[], SlotsType<    {
    default: {
    fields: {
    id: number;
    }[];
    add: () => void;
    remove: (idx: number) => void;
    error: any;
    };
    }>, {
    name: keyof { [K in keyof T as T[K] extends any[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}` | `${K & string}[]` | `${K & string}[${number}]` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`]: any; };
    error?: any;
    ignore?: boolean;
    initialValues?: any[];
    preserve?: boolean;
    default?: any[];
    } & {
    "onUpdate:modelValue"?: ((...args: any[]) => any) | undefined;
    }, PublicProps>;
    Error: DefineSetupFnComponent<    {
    errorFor: keyof { [K in keyof T as T[K] extends any[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}` | `${K & string}[]` | `${K & string}[${number}]` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`]: any; };
    }, {}, SlotsType<    {
    default: {
    error: any;
    };
    }>, {
    errorFor: keyof { [K in keyof T as T[K] extends any[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}` | `${K & string}[]` | `${K & string}[${number}]` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`]: any; };
    } & {}, PublicProps>;
    handleSubmit: (cb?: ((data?: T | undefined) => void | Promise<any>) | undefined) => () => Promise<void>;
    setError: (name: keyof { [K in keyof T as T[K] extends any[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}` | `${K & string}[]` | `${K & string}[${number}]` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`]: any; }, error: any) => void;
    setInitalValues: (initials: Partial<T>) => void;
    setValues: (values: Partial<T>) => void;
    setValue: (name: keyof { [K in keyof T as T[K] extends any[] ? T[K] extends Record<string, any>[] ? `${K & string}[].${GetNestedArray<T[K][0]>}` | `${K & string}[${number}].${GetNestedArray<T[K][0]>}` | `${K & string}` : `${K & string}` | `${K & string}[]` | `${K & string}[${number}]` : T[K] extends Record<string, any> ? `${K & string}.${GetNestedArray<T[K]> & string}` : `${K & string}`]: any; }, value: any) => void;
    reset: () => void;
    isSubmitting: Ref<boolean, boolean>;
    values: [T] extends [Ref<any, any>] ? IfAny<T, Ref<T, T>, T> : Ref<UnwrapRef<T>, T | UnwrapRef<T>>;
};

export { }
